import {
  ChangeDetectionStrategy,
  Component,
  EventEmitter,
  inject,
  Input,
  Output,
  signal,
  WritableSignal,
} from '@angular/core';
import { TransactionService } from '@service/transaction.service';
import { Observable } from 'rxjs';
import { Transaction } from '../../model/Transaction';
import { AsyncPipe, CurrencyPipe, DatePipe, NgForOf, NgIf } from '@angular/common';
import { AnimatedNumberComponent } from '@blackjack/components/animated-number/animated-number.component';

const PER_PAGE = 5

@Component({
  standalone: true,
  selector: 'app-transaction-history',
  imports: [
    NgForOf,
    AsyncPipe,
    CurrencyPipe,
    DatePipe,
    AnimatedNumberComponent,
    NgIf,
  ],
  templateUrl: './transaction-history.component.html',
  styleUrl: './transaction-history.component.css',
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class TransactionHistoryComponent {
  @Input()
  isOpen: boolean = false;
  loading: WritableSignal<boolean> = signal<boolean>(true);
  skeletonItems = Array(PER_PAGE).fill({});
  @Output()
  closeEventEmitter = new EventEmitter<void>();

  private offset: number = 0;

  private transactionService: TransactionService = inject(TransactionService);
  transactions$: Observable<Transaction[]> = this.loadTransactions();

  closeDialog() {
    this.isOpen = false;
    this.closeEventEmitter.emit();
  }

  forward() {
    this.offset++;
    this.transactions$ = this.loadTransactions();
  }

  back() {
    this.offset--;
    this.transactions$ = this.loadTransactions();
  }

  loadTransactions() {
    this.loading.set(true);

    const transactions$ = this.transactionService.getUsersTransactions(PER_PAGE, this.offset * PER_PAGE);

    transactions$.subscribe({complete: () => this.loading.set(false)})

    return transactions$;
  }
}
